#!/usr/bin/env python3

from alpsplot.figure import Figure
import numpy as np
import matplotlib.pyplot as plt
import shutil


def test_save():
    fig = Figure('test_set_title')
    fig.save()
    fig.save(path='./output/abc/c.pdf')
    shutil.rmtree('./output/abc')


def test_set_title():
    fig = Figure('test_set_title')
    fig.set_title()
    fig.set_title('test2')
    fig.save()


def test_set_axis_label():
    fig = Figure('test_set_axis_label')
    fig.set_axis_label('x', r'x - $\alpha$')
    fig.set_axis_label('y', r'y - $\beta$')
    fig.save()


def test_set_axis_lim():
    fig = Figure('test_set_axis_lim')
    fig.set_axis_lim('x')
    fig.set_axis_lim('y', _format='integer')
    fig.save()


def test_plot():
    fig = Figure('test_plot')
    x = np.arange(0, 1, 0.1)
    fig.plot(x, x, label='plot')
    fig.save()


def test_scatter():
    fig = Figure('test_scatter')
    x = np.arange(0, 1, 0.1)
    fig.scatter(x, x, label='scatter')
    fig.scatter(x, x / 2, label='scatter', curve_legend=True)
    fig.save()


def test_bar():
    fig = Figure('test_bar')
    x = np.arange(0, 1, 0.1)
    fig.bar(x, x, width=0.1, label='bar')
    fig.set_legend()
    fig.save()


def test_bar3d():
    _fig = plt.figure(figsize=(5, 2.5))
    _ax = _fig.add_subplot(111, projection='3d')
    fig = Figure('test_bar', fig=_fig, ax=_ax)

    _x = np.arange(0, 1, 0.1)
    _y = np.arange(0, 1, 0.1)
    _xx, _yy = np.meshgrid(_x, _y)
    _xx: np.ndarray
    _yy: np.ndarray
    x, y = _xx.ravel(), _yy.ravel()

    z = (x + y) / 2
    fig.bar3d(x, y, z, size=0.1, label='bar3d')
    fig.save()


def test_set_legend():
    fig = Figure('test_set_legend')
    x = np.arange(0, 1, 0.1)
    fig.plot(x, x, label='plot')
    fig.scatter(x, x, label='scatter')
    fig.set_legend()
    fig.save()


def test_hist():
    fig = Figure('test_hist')
    x = np.arange(0, 1, 0.01)
    fig.hist(x, 5)
    fig.save()


def test_autolabel():
    fig = Figure('test_hist')
    x = np.arange(0, 1, 0.1)
    rects = fig.bar(x, x, width=0.1, label='bar')
    fig.autolabel(rects)
    fig.autolabel(rects, above=False)
    fig.set_legend()
    fig.save()




#!/usr/bin/env python3

from alpsplot.utils import *
import numpy as np


def test_get_roc_curve():
    label = np.arange(10.0)
    pred = np.arange(10.0)
    pred[5:] += 1
    get_roc_curve(label, pred)


def test_normalize():
    x = np.arange(10.0)
    normalize(x)


def test_avg_smooth():
    x = np.arange(10.0)
    avg_smooth(x)


def test_monotone():
    x = np.arange(10.0)
    monotone(x, increase=True)
    monotone(x, increase=False)


def test_gaussian_kde():
    x = np.random.randn(32)
    x_grid = np.arange(-3.0, 3.0)
    gaussian_kde(x, x_grid)


def test_groups_err_bar():
    x = np.array([1, 2, 3, 4, 5]*3)
    y = np.arange(15.0)
    groups_err_bar(x, y)


def test_flatten_err_bar():
    x = np.array([1, 2, 3, 4, 5]*3)
    y = np.arange(15.0)
    y_dict = groups_err_bar(x, y)
    flatten_err_bar(y_dict)


def test_adjust_err_bar():
    x = np.array([1, 2, 3, 4, 5]*3)
    y = np.arange(15.0)
    y_dict = groups_err_bar(x, y)
    adjust_err_bar(y_dict, mean=0.3, std=0.4)


def test_normalize_err_bar():
    x = np.array([1, 2, 3, 4, 5]*3)
    y = np.arange(15.0)
    normalize_err_bar(x, y)


def test_avg_smooth_err_bar():
    x = np.array([1, 2, 3, 4, 5]*3)
    y = np.arange(15.0)
    avg_smooth_err_bar(x, y)



#!/usr/bin/env python3

from .fonts import add_optima, add_palatino

import os
import numpy as np

import matplotlib.ticker as ticker
import matplotlib
from matplotlib import pyplot as plt
from matplotlib.figure import Figure
from matplotlib.axes import Axes
from matplotlib.lines import Line2D
from matplotlib.container import BarContainer
import seaborn

add_optima()
add_palatino()
matplotlib.rc('mathtext', fontset='cm')
matplotlib.rc('pdf', fonttype=42)
matplotlib.rc('ps', fonttype=42)
matplotlib.rc('svg', image_inline=True, fonttype='none')


# Markers
# '.' point marker
# ',' pixel marker
# 'o' circle marker
# 'v' triangle_down marker
# '^' triangle_up marker
# '<' triangle_left marker
# '>' triangle_right marker
# '1' tri_down marker
# '2' tri_up marker
# '3' tri_left marker
# '4' tri_right marker
# 's' square marker
# 'p' pentagon marker
# '*' star marker
# 'h' hexagon1 marker
# 'H' hexagon2 marker
# '+' plus marker
# 'x' x marker
# 'D' diamond marker
# 'd' thin_diamond marker
# '|' vline marker
# '_' hline marker

# Line Styles
# '-'     solid line style
# '--'    dashed line style
# '-.'    dash-dot line style
# ':'     dotted line style

class Figure:
    def __init__(self, name: str, folder_path: str = None, fig: Figure = None, ax: Axes = None, figsize: tuple[float, float] = (5, 2.5), **kwargs):
        super(Figure, self).__init__()
        self.name: str = name
        self.folder_path: str = folder_path
        if folder_path is None:
            self.folder_path = './output/'
        self.fig: Figure = fig
        self.ax: Axes = ax
        if fig is None and ax is None:
            self.fig: Figure = plt.figure(figsize=figsize, **kwargs)
            self.ax = self.fig.add_subplot(1, 1, 1)
        self.ax.spines['top'].set_visible(False)
        self.ax.spines['bottom'].set_visible(True)
        self.ax.spines['left'].set_visible(False)
        self.ax.spines['right'].set_visible(False)
        self.ax.grid(axis='y', linewidth=1, alpha=0.5)
        self.ax.set_axisbelow(True)

        self.ax.set_xlim([0.0, 1.0])
        self.ax.set_ylim([0.0, 1.0])

    def save(self, path: str = None, folder_path: str = None, ext: str = '.pdf') -> None:
        if path is None:
            if folder_path is None:
                folder_path = self.folder_path
            path = os.path.join(folder_path, f'{self.name}{ext}')
        else:
            folder_path = os.path.dirname(path)
        if not os.path.exists(folder_path):
            os.makedirs(folder_path)
        self.fig.savefig(path, dpi=100, bbox_inches='tight')

    def set_title(self, text: str = None, fontsize: int = 16,
                  fontproperties: str = 'Optima', fontweight: str = 'bold', math_fontfamily: str = 'cm') -> None:
        if text is None:
            text = self.name
        self.ax.set_title(text, fontsize=fontsize,
                          fontproperties=fontproperties, fontweight=fontweight, math_fontfamily=math_fontfamily)

    def set_axis_label(self, axis: str, text: str, fontsize: int = 12,
                       fontproperties: str = 'Optima', fontweight='bold', math_fontfamily: str = 'cm', **kwargs):
        func = getattr(self.ax, f'set_{axis}label')
        func(text, fontsize=fontsize, fontproperties=fontproperties,
             fontweight=fontweight, math_fontfamily=math_fontfamily, **kwargs)

    def set_axis_lim(self, axis: str, lim: list[float] = [0.0, 1.0], margin: list[float] = [0.0, 0.0],
                     piece: int = 10, _format: str = '%.1f', fontsize: int = 11,
                     fontproperties: str = 'Optima', fontweight: str = 'bold', math_fontfamily: str = 'cm') -> None:
        if _format == 'integer':
            _format = '%d'
        lim_func = getattr(self.ax, f'set_{axis}lim')
        set_ticks_func = getattr(self.ax, f'set_{axis}ticks')

        def format_func(_str):
            getattr(self.ax, f'{axis}axis').set_major_formatter(
                ticker.FormatStrFormatter(_str))
        ticks = np.append(
            np.arange(lim[0], lim[1], (lim[1] - lim[0]) / piece), lim[1])
        final_lim = [lim[0] - margin[0], lim[1] + margin[1]]
        lim_func(final_lim)
        set_ticks_func(ticks)
        ticks = getattr(self.ax, f'get_{axis}ticks')()
        set_ticklabels_func = getattr(self.ax, f'set_{axis}ticklabels')
        set_ticklabels_func(ticks, fontsize=fontsize,
                            fontproperties=fontproperties, fontweight=fontweight, math_fontfamily=math_fontfamily)
        format_func(_format)

    def set_legend(self, *args, fontsize=11, frameon: bool = True, edgecolor='white', framealpha=1.0,
                   fontproperties: str = 'Optima', fontstyle=None, fontweight='bold',
                   math_fontfamily: str = 'cm', **kwargs) -> None:
        self.ax.legend(*args, frameon=frameon, edgecolor=edgecolor,
                       framealpha=framealpha, **kwargs)
        plt.setp(self.ax.get_legend().get_texts(), fontsize=fontsize,
                 fontproperties=fontproperties, fontstyle=fontstyle, fontweight=fontweight, math_fontfamily=math_fontfamily)

    def plot(self, x: np.ndarray, y: np.ndarray, color: str = 'black', linewidth: int = 2,
             label: str = None, markerfacecolor: str = 'white', linestyle: str = '-', zorder: int = 1, **kwargs) -> Line2D:
        # linestyle marker markeredgecolor markeredgewidth markerfacecolor markersize alpha
        ax = seaborn.lineplot(x=x, y=y, ax=self.ax, color=color, linewidth=linewidth,
                              markerfacecolor=markerfacecolor, zorder=zorder, **kwargs)
        line: Line2D = ax.get_lines()[-1]
        line.set_linestyle(linestyle)
        if label is not None:
            self.curve_legend(label=label, color=color,
                              linewidth=linewidth, linestyle=linestyle, **kwargs)
        return line

    def curve_legend(self, label: str = None, color: str = 'black', linewidth: int = 2, linestyle: str = '-',
                     markerfacecolor: str = 'white', **kwargs) -> Line2D:
        # linestyle marker markeredgecolor markeredgewidth markerfacecolor markersize alpha
        line, = self.ax.plot([], [], color=color, linewidth=linewidth, markeredgewidth=linewidth, markeredgecolor=color,
                             label=label, markerfacecolor=markerfacecolor, linestyle=linestyle, **kwargs)
        return line

    def scatter(self, x: np.ndarray, y: np.ndarray, color: str = 'black', linewidth: int = 2,
                label: str = None, curve_legend: bool = False,
                marker: str = 'D', facecolor: str = 'white', zorder: int = 3, **kwargs):
        # marker markeredgecolor markeredgewidth markerfacecolor markersize alpha
        if curve_legend and label is not None:
            self.curve_legend(label=label, color=color,
                              linewidth=linewidth, marker=marker, **kwargs)
            label = None
        return self.ax.scatter(x=x, y=y, color=color, linewidth=linewidth, label=label, marker=marker, facecolor=facecolor, zorder=zorder, **kwargs)

    def bar(self, x: np.ndarray, y: np.ndarray, color: str = 'black', width: float = 0.2,
            align: str = 'edge', edgecolor: str = 'white', label: str = None, **kwargs) -> BarContainer:
        # facecolor edgewidth alpha
        return self.ax.bar(x, y, color=color, width=width, align=align, edgecolor=edgecolor, label=label, **kwargs)

    def bar3d(self, x: np.ndarray, y: np.ndarray, z: np.ndarray, color: str = 'black', size: tuple[float, float] = 0.5,
              label: str = None, **kwargs) -> BarContainer:
        # facecolor edgewidth alpha
        if isinstance(size, (float, int)):
            size = [size, size]
        return self.ax.bar3d(x=x, y=y, z=np.zeros_like(x),
                             dx=np.ones_like(x) * size[0], dy=np.ones_like(y) * size[1], dz=z,
                             color=color, label=label, **kwargs)

    def hist(self, x: np.ndarray, bins: int = None, density: bool = True, **kwargs):
        return self.ax.hist(x, bins=bins, density=density, **kwargs)

    def autolabel(self, rects: BarContainer, above: bool = True, fontsize: int = 6,
                  fontproperties: str = 'Optima', fontweight: str = 'bold', math_fontfamily: str = 'cm') -> None:
        """Attach a text label above each bar in *rects*, displaying its height."""
        for rect in rects:
            height = int(rect.get_height())
            offset = 3 if above else -13
            self.ax.annotate(f'{int(abs(height)):d}',
                             xy=(rect.get_x() + rect.get_width() / 2, height),
                             xytext=(0, offset),  # 3 points vertical offset
                             textcoords="offset points",
                             ha='center', va='bottom', fontsize=fontsize,
                             fontproperties=fontproperties, fontweight=fontweight, math_fontfamily=math_fontfamily)



#!/usr/bin/env python3

from alpsplot.fit import *


def test_poly_fit():
    x = np.arange(10)
    y = np.sin(x)
    x_grid = np.arange(10, 2)
    poly_fit(x, y, x_grid, degree=3)


def test_tanh_fit():
    x = np.arange(10)
    y = np.sin(x)
    x_grid = np.arange(10, 2)
    tanh_fit(x, y, x_grid, degree=3)


def test_atan_fit():
    x = np.arange(10)
    y = np.sin(x)
    x_grid = np.arange(10, 2)
    atan_fit(x, y, x_grid, degree=3)


def test_exp_fit():
    x = np.arange(10)
    y = np.sin(x)
    x_grid = np.arange(10, 2)
    exp_fit(x, y, x_grid, degree=3, increase=True)
    exp_fit(x, y, x_grid, degree=3, increase=False)


def test_inverse_fit():
    x = np.arange(1, 10)
    y = np.sin(x)
    x_grid = np.arange(2, 10, 2)
    inverse_fit(x, y, x_grid, degree=3)


def test_interp_fit():
    x = np.arange(10)
    y = np.sin(x)
    x_grid = np.arange(10, 2)
    interp_fit(x, y, x_grid, interp_num=5)



#!/usr/bin/env python3

import numpy as np
from scipy.interpolate import UnivariateSpline
# from scipy.optimize import curve_fit

EPS = 1e-5


def poly_fit(x: np.ndarray, y: np.ndarray, x_grid: np.ndarray, degree: int = 1) -> np.ndarray:
    z = np.polyfit(x, y, degree)
    y_grid = np.polyval(z, x_grid)
    return y_grid


def tanh_fit(x: np.ndarray, y: np.ndarray, x_grid: np.ndarray,
             degree: int = 1, mean_bias: float = 0.0, scale_multiplier: float = 1.0) -> np.ndarray:
    mean = (max(y) + min(y)) / 2 + mean_bias
    scale = max(abs(y - mean)) * scale_multiplier + EPS
    fit_data = np.arctanh((y - mean) / scale)
    z = np.polyfit(x, fit_data, degree)
    y_grid = np.tanh(np.polyval(z, x_grid)) * scale + mean
    return y_grid


def atan_fit(x: np.ndarray, y: np.ndarray, x_grid: np.ndarray,
             degree: int = 1, mean_bias: float = 0.0, scale_multiplier: float = 1.0) -> np.ndarray:
    mean = (max(y) + min(y)) / 2 + mean_bias
    scale = max(abs(y - mean)) * scale_multiplier + EPS
    fit_data = np.tan((y - mean) / scale)
    z = np.polyfit(x, fit_data, degree)
    y_grid = np.tanh(np.polyval(z, x_grid)) * scale + mean
    return y_grid


def exp_fit(x: np.ndarray, y: np.ndarray, x_grid: np.ndarray,
            degree: int = 1, increase: bool = True, eps: float = 0.01) -> np.ndarray:
    y_max = max(y)
    y_min = min(y)

    fit_data = np.log(y + eps - y_min) if increase else np.log(y_max + eps - y)
    z = np.polyfit(x, fit_data, degree)
    y_grid = np.exp(np.polyval(z, x_grid))
    if increase:
        y_grid += y_min - eps
    else:
        y_grid = y_max + eps - y_grid
    return y_grid


def inverse_fit(x: np.ndarray, y: np.ndarray, x_grid: np.ndarray,
                degree: int = 1, y_lower_bound: float = 0.0) -> np.ndarray:
    fit_data = 1 / (y - y_lower_bound)
    z = np.polyfit(x, fit_data, degree)
    y_grid = 1 / (np.polyval(z, x_grid)) + y_lower_bound
    return y_grid


def interp_fit(x: np.ndarray, y: np.ndarray, x_grid: np.ndarray, interp_num: int = 20) -> np.ndarray:
    func = UnivariateSpline(x, y, s=interp_num)
    y_grid = func(x_grid)
    return y_grid



#!/usr/bin/env python3

import numpy as np
import scipy.stats as stats


def get_roc_curve(label: np.ndarray, pred: np.ndarray) -> tuple[np.ndarray, np.ndarray]:
    total_inst = len(label)
    total_pos_inst = len(np.where(label == 1)[0])

    assert len(label) == len(pred)
    # true positive rates and false positive rates
    tprs, fprs, thresholds = [], [], []

    # iterate over all positive thresholds
    for threshold in np.unique(pred):
        pred_pos_idx: np.ndarray = np.where(pred >= threshold)[0]
        # number of predicted positive instances
        pred_pos_inst = len(pred_pos_idx)
        # number of true positive instances
        true_pos_inst: int = np.count_nonzero(label[pred_pos_idx])

        tpr = true_pos_inst / total_pos_inst
        fpr = (pred_pos_inst - true_pos_inst) / (total_inst - total_pos_inst)
        tprs.append(tpr)
        fprs.append(fpr)
        thresholds.append(threshold)
    return np.array(fprs), np.array(tprs)


def normalize(x: np.ndarray, _min: float = None, _max: float = None, tgt_min: float = 0.0, tgt_max: float = 1.0) -> np.ndarray:
    _min = x.min() if _min is None else _min
    _max = x.max() if _max is None else _max
    x = (x - _min) / (_max - _min) * (tgt_max - tgt_min) + tgt_min
    return x


def avg_smooth(x: np.ndarray, window: int = 3) -> np.ndarray:
    new_x = np.zeros_like(x)
    for i in range(len(x)):
        if i < window // 2:
            new_x[i] = (x[0] * (window // 2 - i) +
                        np.sum(x[: i + (window + 1) // 2])) / window
        elif i >= len(x) - (window - 1) // 2:
            new_x[i] = (x[-1] * ((window + 1) // 2 - len(x) + i) +
                        np.sum(x[i - window // 2:])) / window
        else:
            new_x[i] = np.mean(x[i - window // 2: i + 1 + (window - 1) // 2])
    return new_x


def monotone(x: np.ndarray, increase: bool = True) -> np.ndarray:
    y = np.copy(x)
    temp: float = min(x) if increase else max(x)
    for i in range(len(x)):
        if ((increase and x[i] < temp) or (not increase and x[i] > temp)):
            y[i] = temp
        else:
            temp = x[i]
    return y


def gaussian_kde(x: np.ndarray, x_grid: np.ndarray) -> np.ndarray:
    kde_func = stats.gaussian_kde(x)
    y_grid = kde_func(x_grid)
    return y_grid


def groups_err_bar(x: np.ndarray, y: np.ndarray) -> dict[float, np.ndarray]:
    y_dict = {}
    for _x in set(x):
        y_dict[_x] = np.array([y[t] for t in range(len(y)) if x[t] == _x])
    return y_dict


def flatten_err_bar(y_dict: dict[float, np.ndarray]) -> tuple[np.ndarray, np.ndarray]:
    x = []
    y = []
    for _x in y_dict.keys():
        for _y in y_dict[_x]:
            x.append(_x)
            y.append(_y)
    return np.array(x), np.array(y)


def adjust_err_bar(y_dict: dict[float, np.ndarray], mean: np.ndarray = None, std: np.ndarray = None) -> dict[float, np.ndarray]:
    sort_keys = np.sort(list(y_dict.keys()))
    if isinstance(mean, float):
        mean = mean * np.ones(len(sort_keys))
    if isinstance(std, float):
        std = std * np.ones(len(sort_keys))
    for i in range(len(sort_keys)):
        key = sort_keys[i]
        if mean is not None and mean[i]:
            y_dict[key] = y_dict[key] + mean[i]
        if std is not None and std[i]:
            y_dict[key] = y_dict[key].mean() \
                + (y_dict[key] - y_dict[key].mean()) * std[i]
    return y_dict


def normalize_err_bar(x: np.ndarray, y: np.ndarray):
    x = normalize(x)
    y_dict = groups_err_bar(x, y)
    y_mean = np.array([y_dict[_x].mean()
                       for _x in np.sort(list(y_dict.keys()))])
    y_norm = normalize(y_mean)
    y_dict = adjust_err_bar(y_dict, y_norm - y_mean)
    return flatten_err_bar(y_dict)


def avg_smooth_err_bar(x: np.ndarray, y: np.ndarray, window: int = 3):
    y_dict = groups_err_bar(x, y)
    y_mean = np.array([y_dict[_x].mean()
                       for _x in np.sort(list(y_dict.keys()))])
    y_smooth = avg_smooth(y_mean, window=window)
    y_dict = adjust_err_bar(y_dict, y_smooth - y_mean)
    return flatten_err_bar(y_dict)



#!/usr/bin/env python3

from alpsplot.fonts import add_optima, add_palatino
import matplotlib.font_manager
import os

from os.path import dirname as d
from os.path import abspath, normpath


def test_add_palatino():
    fontManager: matplotlib.font_manager.FontManager = matplotlib.font_manager.fontManager
    ttflist: list[matplotlib.font_manager.FontEntry] = fontManager.ttflist
    fname_list: list[str] = []
    ttc_path = os.path.join(normpath(d(d(abspath(__file__)))),
                            'fonts', 'Palatino.ttc')
    fontManager.addfont(ttc_path)
    add_palatino()
    for font in ttflist:
        if 'palatino' in font.fname.lower():
            assert 'ttc' not in font.fname.lower()
            assert font.name == 'Palatino'
            fname_list.append(os.path.basename(font.fname))
    style_list = ['Roman', 'Italic', 'Bold', 'BoldItalic']
    for style in style_list:
        assert f'Palatino-{style}.ttf' in fname_list


def test_add_optima():
    fontManager: matplotlib.font_manager.FontManager = matplotlib.font_manager.fontManager
    ttflist: list[matplotlib.font_manager.FontEntry] = fontManager.ttflist
    fname_list: list[str] = []
    ttc_path = os.path.join(normpath(d(d(abspath(__file__)))),
                            'fonts', 'Optima.ttc')
    fontManager.addfont(ttc_path)
    add_optima()
    for font in ttflist:
        if 'optima' in font.fname.lower():
            print(font.fname)
            assert 'ttc' not in font.fname.lower()
            assert font.name == 'Optima'
            fname_list.append(os.path.basename(font.fname))
    style_list = ['Regular', 'Italic', 'Bold', 'BoldItalic', 'ExtraBlack']
    for style in style_list:
        assert f'Optima-{style}.ttf' in fname_list



#!/usr/bin/env python3

import os
import matplotlib
import matplotlib.font_manager


def add_optima() -> None:
    dirname = os.path.dirname(__file__)
    fontManager: matplotlib.font_manager.FontManager = matplotlib.font_manager.fontManager
    ttflist: list[matplotlib.font_manager.FontEntry] = fontManager.ttflist
    for i, font in enumerate(ttflist):
        if 'optima.ttc' in font.fname.lower():
            del ttflist[i]

    for style in ['Regular', 'Italic', 'Bold', 'BoldItalic', 'ExtraBlack']:
        file_path = os.path.normpath(
            os.path.join(dirname, f'Optima-{style}.ttf'))
        fontManager.addfont(file_path)


def add_palatino() -> None:
    dirname = os.path.dirname(__file__)
    fontManager: matplotlib.font_manager.FontManager = matplotlib.font_manager.fontManager
    ttflist: list[matplotlib.font_manager.FontEntry] = fontManager.ttflist
    for i, font in enumerate(ttflist):
        if 'palatino.ttc' in font.fname.lower():
            del ttflist[i]

    for style in ['Roman', 'Italic', 'Bold', 'BoldItalic']:
        file_path = os.path.normpath(
            os.path.join(dirname, f'Palatino-{style}.ttf'))
        fontManager.addfont(file_path)
